@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 // DO NOT DEFINE STATIC DATA IN THIS HEADER!
@@ -498,8 +499,8 @@ static inline Vectorized<T> binary_fp8_op_as_fp32(
 
 // Refer to
 // https://github.com/pytorch/pytorch/pull/153364#discussion_r2086509353 FP8 +,
-// -, *, /, planed to be deleted in the future and here is just to make compiler
-// happy
+// -, *, /, planned to be deleted in the future and here is just to make
+// compiler happy
 Vectorized<Float8_e4m3fn> inline operator+(
     const Vectorized<Float8_e4m3fn>& a,
     const Vectorized<Float8_e4m3fn>& b) {
@@ -585,8 +586,8 @@ class Vectorized<Float8_e5m2> : public Vectorizedf8<Float8_e5m2> {
 
 // Refer to
 // https://github.com/pytorch/pytorch/pull/153364#discussion_r2086509353 FP8 +,
-// -, *, /, planed to be deleted in the future and here is just to make compiler
-// happy
+// -, *, /, planned to be deleted in the future and here is just to make
+// compiler happy
 Vectorized<Float8_e5m2> inline operator+(
     const Vectorized<Float8_e5m2>& a,
     const Vectorized<Float8_e5m2>& b) {
@@ -659,3 +660,7 @@ inline Vectorized<Float8_e5m2> Vectorized<Float8_e5m2>::le(
 
 } // namespace CPU_CAPABILITY
 } // namespace at::vec
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
