@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #ifndef C10_UTIL_TYPE_H_
 #define C10_UTIL_TYPE_H_
 
@@ -28,3 +29,7 @@ inline const char* demangle_type() {
 } // namespace c10
 
 #endif // C10_UTIL_TYPE_H_
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
