@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #ifndef CAFFE2_UTILS_THREADPOOL_COMMON_H_
 #define CAFFE2_UTILS_THREADPOOL_COMMON_H_
 
@@ -18,3 +19,7 @@
 #endif // ANDROID / IOS
 
 #endif  // CAFFE2_UTILS_THREADPOOL_COMMON_H_
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
