@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <torch/csrc/Export.h>
 #include <torch/csrc/python_headers.h>
@@ -24,3 +25,7 @@ inline bool THPLayout_Check(PyObject* obj) {
 PyObject* THPLayout_New(at::Layout layout, const std::string& name);
 
 void THPLayout_init(PyObject* module);
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
