@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/Export.h>
@@ -16,3 +17,7 @@ TORCH_API std::pair<std::unique_ptr<Function>, const std::string> aotCompile(
     const std::string& kernel_func_name = "func");
 
 } // namespace torch::jit::mobile::nnc
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
