@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/jit/ir/ir.h>
@@ -14,3 +15,7 @@ TORCH_API void NormalizeOps(const std::shared_ptr<Graph>& graph);
 const std::unordered_map<Symbol, Symbol>& getOperatorAliasMap();
 
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
