@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/jit/ir/ir.h>
@@ -34,3 +35,7 @@ namespace torch::jit {
 TORCH_API bool PeepholeOptimizeDictIdioms(const std::shared_ptr<Graph>& graph);
 
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
