@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 #include <ATen/core/List.h>
 #include <ATen/core/functional.h>
@@ -39,3 +40,7 @@ void tupleSlice(Stack& stack, size_t begin, size_t end);
 void dequantize(Stack& stack);
 
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
