@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/csrc/profiler/orchestration/observer.h>
@@ -12,3 +13,7 @@ using torch::profiler::impl::ProfilerConfig;
 using torch::profiler::impl::profilerEnabled;
 using torch::profiler::impl::ProfilerState;
 } // namespace torch::autograd::profiler
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
