@@ -437,7 +437,7 @@ def pre_load_state_dict_hook(
     Pre-load state dict hook to add ShardedTensor to the module.
     """
     for submodule_name, submodule in module.named_modules():
-        for attr_name in submodule.__dict__.keys():
+        for attr_name in submodule.__dict__:
             mod_prefix = prefix + submodule_name
             key = mod_prefix + ("." if mod_prefix else "") + attr_name
             if key in state_dict:
