@@ -5,11 +5,12 @@ import logging
 import operator
 import re
 from collections import defaultdict
+from collections.abc import Callable
 from contextlib import contextmanager
 from copy import deepcopy
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, Callable, cast, Optional, Union
+from typing import Any, cast
 
 import torch
 import torch.fx._pytree as fx_pytree
@@ -80,7 +81,7 @@ def _disable_interpreter():
 # Assign attribute 'from_obj' to the qualified name 'target' on 'to_module
 # This installs empty Modules where none exist yet if they are subpaths of target
 def _assign_attr(
-    from_obj: Union[torch.Tensor, torch.ScriptObject, torch.nn.Module],
+    from_obj: torch.Tensor | torch.ScriptObject | torch.nn.Module,
     to_module: torch.nn.Module,
     target: str,
     attr_kind: _AttrKind,
@@ -131,9 +132,9 @@ def _assign_attr(
 
 
 class _SubmoduleBase:
-    _ty: Optional[str]
+    _ty: str | None
 
-    def type_name(self) -> Optional[str]:
+    def type_name(self) -> str | None:
         """
         Subclass of this class - InterpreterModule, InterpreterModuleDispatcher, represents
         corresponding model in eager model. To get this type information for those modules
@@ -148,12 +149,12 @@ class InterpreterModule(_SubmoduleBase, torch.nn.Module):
     and makes it easier to debug execution.
     """
 
-    graph_module: Optional[torch.fx.GraphModule]
+    graph_module: torch.fx.GraphModule | None
 
     def __init__(
         self,
         graph: torch.fx.Graph,
-        ty: Optional[str] = None,
+        ty: str | None = None,
     ):
         super().__init__()
         self.graph = graph
@@ -289,8 +290,8 @@ class FlatArgsAdapter(abc.ABC):
         target_spec: pytree.TreeSpec,
         input_spec: pytree.TreeSpec,
         input_args: list[Any],
-        metadata: Optional[dict[str, Any]] = None,
-        obj: Optional[Any] = None,
+        metadata: dict[str, Any] | None = None,
+        obj: Any | None = None,
     ) -> list[Any]:
         """NOTE: This adapter may mutate given ``input_args_with_path``."""
         ...
@@ -300,11 +301,11 @@ class FlatArgsAdapter(abc.ABC):
         return []
 
 
-class UnflattenedModule(torch.nn.Module):
+class UnflattenedModule(_SubmoduleBase, torch.nn.Module):
     def __init__(
         self,
         export_module: ExportedProgram,
-        flat_args_adapter: Optional[FlatArgsAdapter] = None,
+        flat_args_adapter: FlatArgsAdapter | None = None,
     ):
         super().__init__()
         if export_module.graph_signature.backward_signature is not None:
@@ -339,6 +340,7 @@ class UnflattenedModule(torch.nn.Module):
 
         _inplace_buffer_and_input_mutations(export_graph, self.graph_signature)
         _fix_nn_module_stacks(export_graph)
+        self._ty = _root_module_type(export_graph)
 
         self.ivals = _IVals()
         # for any intermediate value of a mutation that is read, track the mutation
@@ -366,7 +368,7 @@ class UnflattenedModule(torch.nn.Module):
         state_dict = export_module.state_dict
         assigned_params: set[str] = set()  # tracking unused params
         id_to_param: dict[
-            Union[int, _TensorID], torch.nn.Parameter
+            int | _TensorID, torch.nn.Parameter
         ] = {}  # handling weight-sharing
         for name in self.graph_signature.parameters:  # this loop adds used params
             param = state_dict[name]
@@ -385,7 +387,7 @@ class UnflattenedModule(torch.nn.Module):
 
         non_persistent_buffers = set(self.graph_signature.non_persistent_buffers)
         assigned_buffers: set[str] = set()  # tracking unused buffers
-        id_to_buffer: dict[Union[int, _TensorID], tuple[torch.nn.Parameter, bool]] = {}
+        id_to_buffer: dict[int | _TensorID, tuple[torch.nn.Parameter, bool]] = {}
         for name in self.graph_signature.buffers:  # this loop adds used buffers
             if name in non_persistent_buffers:
                 persistent = False
@@ -444,9 +446,7 @@ class UnflattenedModule(torch.nn.Module):
                 )
 
         # use id map so we don't double-clone aliased constants
-        id_to_const: dict[
-            Union[int, _TensorID], Union[torch.Tensor, torch._C.ScriptObject]
-        ] = {}
+        id_to_const: dict[int | _TensorID, torch.Tensor | torch._C.ScriptObject] = {}
         for fqn, constant in export_module.constants.items():
             if _id(constant) not in id_to_const:
                 if isinstance(constant, torch.Tensor):
@@ -462,9 +462,7 @@ class UnflattenedModule(torch.nn.Module):
 
         # This is to handle parameters/buffers that point to the same tensor
         # object id -> list of (node_name, target_name)
-        consts_map: dict[Union[int, _TensorID], list[tuple[str, str]]] = defaultdict(
-            list
-        )
+        consts_map: dict[int | _TensorID, list[tuple[str, str]]] = defaultdict(list)
         consts_targets: set[str] = set()
 
         def add_to_consts_map(obj_id, node_name, target_name):
@@ -750,7 +748,7 @@ class UnflattenedModule(torch.nn.Module):
 
 
 def unflatten(
-    module: ExportedProgram, flat_args_adapter: Optional[FlatArgsAdapter] = None
+    module: ExportedProgram, flat_args_adapter: FlatArgsAdapter | None = None
 ) -> UnflattenedModule:
     """Unflatten an ExportedProgram, producing a module with the same module
     hierarchy as the original eager module. This can be useful if you are trying
@@ -857,6 +855,17 @@ def _inplace_buffer_and_input_mutations(
     output_node.args = ((user_outputs),)
 
 
+def _root_module_type(graph: torch.fx.Graph) -> str | None:
+    for node in graph.nodes:
+        if "nn_module_stack" not in node.meta:
+            continue
+
+        for path, ty in node.meta["nn_module_stack"].values():
+            if not path:
+                return ty
+    return None
+
+
 def _fix_nn_module_stacks(graph):
     # For each nn module stack in the graph, check if the fqns in it represent a stack:
     # 1. Each fqn must be a prefix of the next fqn.
@@ -936,6 +945,7 @@ def _check_graph_equivalence(x: torch.nn.Module, y: torch.nn.Module):
                 for key, value in pytree.tree_map(arg_dump, node.kwargs).items()
             ]
             target = node.target if node.op in ("call_function", "get_attr") else ""
+            # pyrefly: ignore [bad-argument-type]
             ret.append(f"{i}: {node.op}[{target}]({', '.join(args_dump)})")
             nodes_idx[id(node)] = i
         return "\n".join(ret)
@@ -961,7 +971,7 @@ def _generate_flatten(gm: torch.fx.GraphModule, node) -> torch.fx.Node:
 
 
 def _generate_flatten_spec(
-    gm: Union[torch.fx.GraphModule, InterpreterModule, UnflattenedModule], node, spec
+    gm: torch.fx.GraphModule | InterpreterModule | UnflattenedModule, node, spec
 ) -> torch.fx.Node:
     name = _add_spec(gm, spec)
     spec_node = gm.graph.get_attr(name)
@@ -969,7 +979,7 @@ def _generate_flatten_spec(
 
 
 def _generate_unflatten(
-    gm: Union[torch.fx.GraphModule, InterpreterModule, UnflattenedModule], nodes, spec
+    gm: torch.fx.GraphModule | InterpreterModule | UnflattenedModule, nodes, spec
 ) -> torch.fx.Node:
     name = _add_spec(gm, spec)
     spec_node = gm.graph.get_attr(name)
@@ -997,7 +1007,7 @@ def _add_submodule(
     mod: torch.nn.Module,
     target: str,
     module_to_add: torch.nn.Module,
-    create_module: Optional[Callable[[str], torch.nn.Module]] = None,
+    create_module: Callable[[str], torch.nn.Module] | None = None,
 ):
     *prefix, field = target.split(".")
 
@@ -1040,10 +1050,10 @@ class _ModuleFrame:
         seen_attrs,
         created_modules,
         parent,
-        module_stack: list[tuple[str, Optional[str], int]],
+        module_stack: list[tuple[str, str | None, int]],
         module_id,
         module_call_graph: dict[str, ModuleCallSignature],
-        module: Optional[Union[torch.fx.GraphModule, UnflattenedModule]] = None,
+        module: torch.fx.GraphModule | UnflattenedModule | None = None,
     ):
         self.flat_graph = flat_graph
         self.nodes = nodes
@@ -1062,7 +1072,7 @@ class _ModuleFrame:
         # generate call name for self.fqn
         self.child_fqn = _call_name(self.fqn, num_calls + 1)
 
-        self.module: Union[torch.fx.GraphModule, UnflattenedModule, InterpreterModule]
+        self.module: torch.fx.GraphModule | UnflattenedModule | InterpreterModule
         if module is not None:
             self.module = module
             self.ivals = module.ivals if hasattr(module, "ivals") else {}  # type: ignore[var-annotated]
@@ -1079,7 +1089,7 @@ class _ModuleFrame:
         self.node_map: dict[torch.fx.Node, torch.fx.Node] = {}
         self.node_to_placeholder = {}
 
-        self.parent_call_module: Optional[torch.fx.Node] = None
+        self.parent_call_module: torch.fx.Node | None = None
         if parent is not None:
             accessor = _compute_accessor(parent.fqn, self.child_fqn)
 
@@ -1110,10 +1120,10 @@ class _ModuleFrame:
         signature = module_call_graph.get(self.child_fqn)
         if signature is not None and self.parent is not None:
             assert signature.in_spec.num_children == 2
-            args_spec = signature.in_spec.children_specs[0]
-            kwargs_spec = signature.in_spec.children_specs[1]
-            assert args_spec.context is None
-            assert kwargs_spec.context is not None
+            assert signature.in_spec.type is tuple
+            args_spec, kwargs_spec = signature.in_spec.children()
+            assert args_spec.type is tuple
+            assert kwargs_spec.type is dict
 
             with self.graph.inserting_after(None):
                 arg_nodes = [
@@ -1149,7 +1159,7 @@ class _ModuleFrame:
                         )
 
             with self.parent.graph.inserting_before(self.parent_call_module):
-                input_nodes: list[Optional[torch.fx.Node]] = []
+                input_nodes: list[torch.fx.Node | None] = []
                 for input in signature.inputs:
                     if isinstance(input, ConstantArgument):
                         input_nodes.append(input.value)  # type: ignore[arg-type]
@@ -1192,6 +1202,7 @@ class _ModuleFrame:
                     for k in kwargs_spec.context
                 }
             assert self.parent_call_module is not None
+            # pyrefly: ignore [bad-assignment]
             self.parent_call_module.args = tuple(arg_nodes)
             self.parent_call_module.kwargs = kwarg_nodes  # type: ignore[assignment]
 
@@ -1263,6 +1274,27 @@ class _ModuleFrame:
                 f"Could not run remap_input() on op type: {x.op} for node {x}"
             )
 
+    def uplift_common_custom_metadata(self) -> None:
+        # Copy custom metadata if all nodes have same custom metadata
+        custom_meta = None
+        for node in self.node_map.values():
+            curr_meta = node.meta.get("custom", {})
+            if custom_meta is None:
+                # first node
+                custom_meta = curr_meta
+                continue
+
+            if curr_meta != custom_meta:
+                custom_meta = {}
+                break
+
+        if custom_meta:
+            # Lift common custom metadata to parent node and clear children node's custom metadata
+            assert self.parent_call_module is not None
+            self.parent_call_module.meta["custom"] = custom_meta
+            for node in self.node_map.values():
+                del node.meta["custom"]
+
     def finalize_outputs(self):
         self.created_modules.pop(self.fqn, None)
 
@@ -1309,14 +1341,14 @@ class _ModuleFrame:
                 tuple(get_actual_output_node(output) for output in orig_outputs),
                 signature.out_spec,
             )
-            parent_out: Optional[torch.fx.Node] = _generate_flatten_spec(
+            parent_out: torch.fx.Node | None = _generate_flatten_spec(
                 self.parent.module, self.parent_call_module, signature.out_spec
             )
-            graph_outputs: Union[torch.fx.Node, list[torch.fx.Node]] = tree_out_node
+            graph_outputs: torch.fx.Node | list[torch.fx.Node] = tree_out_node
         else:
             graph_outputs = []
             # Iterate through nodes we have copied into self.graph.
-            for orig_node in self.node_map.keys():
+            for orig_node in self.node_map:
                 for user_node in orig_node.users:
                     if user_node.name not in self.seen_nodes:
                         # external user node, need to expose as an output
@@ -1341,6 +1373,7 @@ class _ModuleFrame:
             if isinstance(graph_outputs, torch.fx.Node)
             else [o.meta.get("val") for o in graph_outputs]
         )
+        self.uplift_common_custom_metadata()
 
         if len(orig_outputs) == 1 and signature is None:
             self.parent.node_map[orig_outputs[0]] = parent_out
@@ -1379,6 +1412,7 @@ class _ModuleFrame:
 
     def print(self, *args, **kwargs):
         if self.verbose:
+            # pyrefly: ignore [not-iterable]
             print(*args, **kwargs)
 
     def run_from(self, node_idx):
@@ -1472,6 +1506,7 @@ class _ModuleFrame:
                 self.seen_attrs[self.child_fqn].add(node.target)
 
             self.copy_node(node)
+            # pyrefly: ignore [unsupported-operation]
             node_idx += 1
 
 
@@ -1657,7 +1692,7 @@ def _sink_params(
     module: torch.nn.Module,
     inputs_to_state: dict[str, list[str]],
     scope: list[str],
-    module_id_to_inputs_removed: Optional[dict[int, set[str]]] = None,
+    module_id_to_inputs_removed: dict[int, set[str]] | None = None,
 ):
     """Sink params, buffers, and constants from graph inputs into get_attr nodes.
 
