@@ -1,6 +1,5 @@
 # mypy: allow-untyped-defs
 import warnings
-from typing import Optional
 
 import torch
 import torch.nn.functional as F
@@ -261,8 +260,8 @@ class Hardtanh(Module):
         min_val: float = -1.0,
         max_val: float = 1.0,
         inplace: bool = False,
-        min_value: Optional[float] = None,
-        max_value: Optional[float] = None,
+        min_value: float | None = None,
+        max_value: float | None = None,
     ) -> None:
         super().__init__()
         if min_value is not None:
@@ -1053,7 +1052,7 @@ class Softshrink(Module):
         return str(self.lambd)
 
 
-def _check_arg_device(x: Optional[torch.Tensor]) -> bool:
+def _check_arg_device(x: torch.Tensor | None) -> bool:
     if x is not None:
         return x.device.type in [
             "cpu",
@@ -1063,7 +1062,7 @@ def _check_arg_device(x: Optional[torch.Tensor]) -> bool:
     return True
 
 
-def _arg_requires_grad(x: Optional[torch.Tensor]) -> bool:
+def _arg_requires_grad(x: torch.Tensor | None) -> bool:
     if x is not None:
         return x.requires_grad
     return False
@@ -1074,9 +1073,13 @@ def _is_make_fx_tracing():
         torch_dispatch_mode_stack = (
             torch.utils._python_dispatch._get_current_dispatch_mode_stack()
         )
-        return any(
-            type(x) == torch.fx.experimental.proxy_tensor.ProxyTorchDispatchMode
-            for x in torch_dispatch_mode_stack
+        # this can be triggered when dynamo inlining the module too.
+        return (
+            any(
+                type(x) is torch.fx.experimental.proxy_tensor.ProxyTorchDispatchMode
+                for x in torch_dispatch_mode_stack
+            )
+            or torch.compiler.is_exporting()
         )
     else:
         return False
@@ -1152,8 +1155,8 @@ class MultiheadAttention(Module):
     """
 
     __constants__ = ["batch_first"]
-    bias_k: Optional[torch.Tensor]
-    bias_v: Optional[torch.Tensor]
+    bias_k: torch.Tensor | None
+    bias_v: torch.Tensor | None
 
     def __init__(
         self,
@@ -1254,12 +1257,12 @@ class MultiheadAttention(Module):
         query: Tensor,
         key: Tensor,
         value: Tensor,
-        key_padding_mask: Optional[Tensor] = None,
+        key_padding_mask: Tensor | None = None,
         need_weights: bool = True,
-        attn_mask: Optional[Tensor] = None,
+        attn_mask: Tensor | None = None,
         average_attn_weights: bool = True,
         is_causal: bool = False,
-    ) -> tuple[Tensor, Optional[Tensor]]:
+    ) -> tuple[Tensor, Tensor | None]:
         r"""Compute attention outputs using query, key, and value embeddings.
 
             Supports optional parameters for padding, masks and attention weights.
@@ -1513,10 +1516,10 @@ class MultiheadAttention(Module):
 
     def merge_masks(
         self,
-        attn_mask: Optional[Tensor],
-        key_padding_mask: Optional[Tensor],
+        attn_mask: Tensor | None,
+        key_padding_mask: Tensor | None,
         query: Tensor,
-    ) -> tuple[Optional[Tensor], Optional[int]]:
+    ) -> tuple[Tensor | None, int | None]:
         r"""Determine mask type and combine masks if necessary.
 
         If only one mask is provided, that mask
@@ -1531,8 +1534,8 @@ class MultiheadAttention(Module):
             merged_mask: merged mask
             mask_type: merged mask type (0, 1, or 2)
         """
-        mask_type: Optional[int] = None
-        merged_mask: Optional[Tensor] = None
+        mask_type: int | None = None
+        merged_mask: Tensor | None = None
 
         if key_padding_mask is not None:
             mask_type = 1
@@ -1728,9 +1731,9 @@ class Softmin(Module):
     """
 
     __constants__ = ["dim"]
-    dim: Optional[int]
+    dim: int | None
 
-    def __init__(self, dim: Optional[int] = None) -> None:
+    def __init__(self, dim: int | None = None) -> None:
         super().__init__()
         self.dim = dim
 
@@ -1793,9 +1796,9 @@ class Softmax(Module):
     """
 
     __constants__ = ["dim"]
-    dim: Optional[int]
+    dim: int | None
 
-    def __init__(self, dim: Optional[int] = None) -> None:
+    def __init__(self, dim: int | None = None) -> None:
         super().__init__()
         self.dim = dim
 
@@ -1878,9 +1881,9 @@ class LogSoftmax(Module):
     """
 
     __constants__ = ["dim"]
-    dim: Optional[int]
+    dim: int | None
 
-    def __init__(self, dim: Optional[int] = None) -> None:
+    def __init__(self, dim: int | None = None) -> None:
         super().__init__()
         self.dim = dim
 
